package com.framsticks.dumping;

import com.framsticks.core.LocalInstance;
import com.framsticks.core.Path;
import com.framsticks.core.Instance;
import com.framsticks.util.dispatching.Future;

import org.apache.commons.configuration.Configuration;
import org.apache.log4j.Logger;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

/**
 * @author Piotr Sniegowski
 */
public class FileInstance extends LocalInstance {

	private static final Logger log = Logger.getLogger(Instance.class.getName());
	protected File file;

	public FileInstance() {
	}

	@Override
	public void configure(Configuration config) {
		super.configure(config);
		file = new File(config.getString("filename"));
	}

	@Override
	public void run() {
		assert isActive();
		super.run();
		try {
			FileReader fileReader = new FileReader(file);
			LoadStream stream = new LoadStream(this.getRootPath(), new BufferedReader(fileReader), this, new Future<Path>() {
				@Override
				public void result(Path result, Exception e) {
					if (e != null) {
						log.error("failed to load file instance " + FileInstance.this + ": " + e);
						fireRun(e);
						return;
					}
					log.info("loaded file instance " + FileInstance.this);
					fireRun(null);
				}
			});
			stream.load();
		} catch (IOException e) {
			log.error("io failure: " + e);
			fireRun(e);
		}
	}

	@Override
	public String toString() {
		return "file@" + file.getName();
	}



}
